<?php

namespace Drupal\jsonapi_include;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\NestedArray;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * The class for parse Jsonapi content.
 *
 * @package Drupal\jsonapi_include
 */
class JsonapiParse implements JsonapiParseInterface {

  /**
   * The variable store included data.
   *
   * @var array
   */
  protected array $included;

  /**
   * Allowed includes.
   *
   * @var array
   */
  protected array $allowed;

  public function __construct(
    protected RequestStack $requestStack,
  ) {}

  /**
   * Standard create function.
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('request_stack'),
    );
  }

  /**
   * {@inheritdoc}
   */
  #[\Override]
  public function parse(Response $response): Response {
    $this->parseJsonContent($response);
    return $response;
  }

  /**
   * Check array is assoc.
   *
   * @param array $arr
   *   The array.
   *
   * @return bool
   *   Check result.
   */
  protected function isAssoc(array $arr): bool {
    if ([] === $arr) {
      return FALSE;
    }
    return array_keys($arr) !== range(0, count($arr) - 1);
  }

  /**
   * Group include.
   *
   * @param object|array $object
   *   The input data.
   *
   * @return array
   *   The includes data.
   */
  protected function groupIncludes(object|array $object): array {
    $result = [];
    $included = empty($object['included']) ? [] : $object['included'];
    array_walk($included, function (array $resource, $index) use (&$result): void {
      $result[$resource['type']][$resource['id']] = $resource;
    });
    return $result;
  }

  /**
   * Resolve attributes.
   *
   * @param mixed $item
   *   The input item.
   *
   * @return array
   *   The resolve output.
   */
  protected function resolveAttributes(mixed $item): array {
    $resource = $item;
    if (!empty($resource['attributes'])) {
      foreach ($resource['attributes'] as $name => $value) {
        $resource[$name] = $value;
      }
      unset($resource['attributes']);
    }
    return $resource;
  }

  /**
   * Flatten included.
   *
   * @param mixed $resource
   *   The resource.
   * @param string $key
   *   The key.
   *
   * @return array
   *   The result.
   */
  protected function flattenIncluded(mixed $resource, string $key): array {
    if (isset($this->included[$resource['type']][$resource['id']])) {
      $object = $this->resolveAttributes($this->included[$resource['type']][$resource['id']]);
      if (isset($resource['meta'])) {
        $object['meta'] = $resource['meta'];
      }
    }
    else {
      $object = $resource;
    }
    $result = $this->resolveRelationships($object, $key);
    return $result;
  }

  /**
   * Check resource is include.
   *
   * @param mixed $resource
   *   The resource to verify.
   * @param string $key
   *   Relationship key.
   *
   * @return bool
   *   Check result.
   */
  protected function isIncluded(mixed $resource, string $key): bool {
    if (!in_array($key, $this->allowed) && count(preg_grep('/^' . preg_quote($key) . '\..*/', $this->allowed)) === 0) {
      return FALSE;
    }

    return isset($resource['type']) && isset($this->included[$resource['type']]);
  }

  /**
   * Resolve data.
   *
   * @param mixed $data
   *   The data for resolve.
   * @param string $key
   *   Relationship key.
   *
   * @return array
   *   Result.
   */
  protected function resolveData(mixed $data, string $key): array {
    if ($this->isIncluded($data, $key)) {
      return $this->flattenIncluded($data, $key);
    }
    else {
      return $data;
    }
  }

  /**
   * Resolve data.
   *
   * @param mixed $links
   *   The data for resolve.
   * @param string $key
   *   Relationship key.
   *
   * @return array
   *   Result.
   */
  protected function resolveRelationshipData(mixed $links, string $key): array {
    if (empty($links['data'])) {
      return $links;
    }
    $output = [];
    if (!$this->isAssoc($links['data'])) {
      foreach ($links['data'] as $item) {
        $output[] = $this->resolveData($item, $key);
      }
    }
    else {
      $output = $this->resolveData($links['data'], $key);
    }
    return $output;
  }

  /**
   * Resolve relationships.
   *
   * @param mixed $resource
   *   The data for resolve.
   * @param string $parent_key
   *   The parent key for relationship.
   *
   * @return array
   *   Result.
   */
  protected function resolveRelationships(mixed $resource, string $parent_key): array {
    if (empty($resource['relationships'])) {
      return $resource;
    }

    foreach ($resource['relationships'] as $key => $value) {
      $resource[$key] = $this->resolveRelationshipData($value, trim("$parent_key.$key", '.'));
    }
    unset($resource['relationships']);
    return $resource;
  }

  /**
   * Parse Resource.
   *
   * @param mixed $item
   *   The data for resolve.
   *
   * @return array
   *   Result.
   */
  protected function parseResource(mixed $item): array {
    $attributes = $this->resolveAttributes($item);
    return $this->resolveRelationships($attributes, '');
  }

  /**
   * Integrates includes into the content of a Response.
   *
   * @param \Symfony\Component\HttpFoundation\Response|string|array $response
   *   A Response object or string/array with a response content.
   *
   * @return \Symfony\Component\HttpFoundation\Response|array
   *   Returns an array with the response content, if the input is string or
   *   array, or void if input is a Response.
   */
  protected function parseJsonContent(Response $response): Response|array {
    $content = $response->getContent();
    if (is_string($content)) {
      $content = Json::decode($content);
    }

    if (NestedArray::getValue($content, ['jsonapi', 'parsed'])) {
      return $response;
    }
    if (isset($content['errors']) || empty($content['data'])) {
      return $response;
    }
    $this->included = $this->groupIncludes($content);
    $include_parameter = $this->requestStack->getCurrentRequest()->query->get('include');
    $this->allowed = array_map('trim', explode(',', $include_parameter ?? ''));
    $data = [];
    if (!$this->isAssoc($content['data'])) {
      foreach ($content['data'] as $item) {
        $data[] = $this->parseResource($item);
      }
    }
    else {
      $data = $this->parseResource($content['data']);
    }
    if (isset($content['included'])) {
      unset($content['included']);
    }
    $content['jsonapi']['parsed'] = TRUE;
    $content['data'] = $data;
    $response->setContent(Json::encode($content));
    return $response;
  }

}
